@@ -9,7 +9,7 @@ from typing import Union
 import yaml
 from friendlylog import colored_logger as log
 
-from python_rules_evaluator.modules.jsongeek import dotpath_get_value
+from python_rules_evaluator.modules.jsongeek import dotpath_get_value, dotpath_exists
 
 
 class Evaluator():
@@ -65,10 +65,11 @@ class Evaluator():
                 elif var in context['consts']:
                     expr = re.sub(
                         f"\\${var}", f"{context['consts'][var]}{' (' + var + ')' if informative else ''}", expr)
-                elif 'fields' in context and dotpath_get_value(context['fields'], var) is not None:
+                elif 'fields' in context and dotpath_exists(context['fields'], var):
                     expr = re.sub(
                         f"\\${var}", f"{dotpath_get_value(context['fields'], var)}{' (' + var + ')' if informative else ''}", expr)
                 else:
+                    # print(context['fields'])
                     log.warning(
                         f"Variable '{var}' is not defined, used in expression '{expr}'.")
                     expr = re.sub(f"\\${var}", 'None', expr)
@@ -89,8 +90,11 @@ class Evaluator():
 
         funcs = {}
         exec(f"def func(): {lines}", self.functions, funcs)
-
-        return funcs['func']()
+        try:
+            result = funcs['func']()
+        except Exception as ex:
+            log.error(f"Cannot execute function\n===\n{body}\n===\n{ex}")
+        return result
 
     def __eval_and_or(self, condition: Union[dict, list, str], context: dict):
         evaluation = ''
@@ -154,6 +158,7 @@ class Evaluator():
         for var, fieldpath in _vars.items():
             if (isinstance(fieldpath, str)):
                 vars[var] = dotpath_get_value(obj, fieldpath)
+                # print(f"{var} = {vars[var]}")
             if (isinstance(fieldpath, dict)):
                 if 'func' in _vars[var]:
                     vars[var] = self.__exec_function(
@@ -228,7 +233,6 @@ class Evaluator():
             log.debug(f"{key}: {val}")
 
     def __get_evaluation(self, evaluation, context):
-
         return json.loads(self.__replace_vars(
             json.dumps(evaluation), context, informative=True))
 
